@@ -144,16 +144,17 @@ get_expression(GCDisplay *display)
 gboolean
 display_get_integer(GCDisplay *display, gint64 *value)
 {
-    gchar *endptr;
+    MPNumber t, min, max;
 
-    if (display_is_result(display)) {
-        *value = mp_cast_to_int(display_get_answer(display));
-        return TRUE;
-    }
-
-    *value = g_ascii_strtoll(get_text(display), &endptr, 10);
-    if(*endptr != '\0' || ((*value == G_MAXINT64 || *value == G_MININT64) && errno == ERANGE))
+    if (!display_is_usable_number(display, &t))
         return FALSE;
+
+    mp_set_from_integer(G_MININT64, &min);
+    mp_set_from_integer(G_MAXINT64, &max);
+    if (mp_is_less_than(&t, &min) || mp_is_greater_than(&t, &max))
+        return FALSE;
+
+    *value = mp_cast_to_int(&t);
     return TRUE;
 }
 
@@ -161,23 +162,17 @@ display_get_integer(GCDisplay *display, gint64 *value)
 gboolean
 display_get_unsigned_integer(GCDisplay *display, guint64 *value)
 {
-    const char *text;
-    char buf[MAX_DISPLAY];
-    gchar *endptr;
+    MPNumber t, max;
 
-    text = get_text(display);
-    if (display_is_result(display)) {
-        display_make_number(display, buf, MAX_DISPLAY, display_get_answer(display));
-        text = buf;
-    }
-
-    /* strtoull() treats the string like a 2's complement number which is not what we want */
-    if(strncmp(text, "-", strlen("-")) == 0 || strncmp(text, "−", strlen("−")) == 0)
+    if (!display_is_usable_number(display, &t))
+        return FALSE;
+  
+    mp_set_from_unsigned_integer(G_MAXUINT64, &max);
+    char string[MAX_DIGITS];
+    if (mp_is_negative(&t) || mp_is_greater_than(&t, &max))
         return FALSE;
 
-    *value = g_ascii_strtoull(text, &endptr, 10);
-    if(*endptr != '\0' || (*value == G_MAXUINT64 && errno == ERANGE))
-        return FALSE;
+    *value = mp_cast_to_unsigned_int(&t);
     return TRUE;
 }
 
@@ -295,6 +290,22 @@ display_set_error(GCDisplay *display, const char *message)
 }
 
 
+void
+display_convert(GCDisplay *display, DisplayFormat format)
+{
+    DisplayFormat old_format;
+
+    if (!display_is_result (display))
+        return;
+
+    /* FIXME: A bit hacky... */
+    old_format = display->format;
+    display->format = format;
+    display_set_answer(display);
+    display->format = old_format;
+}
+
+
 static void
 copy_state(GCDisplayState *dst, GCDisplayState *src)
 {
@@ -547,6 +558,32 @@ display_is_usable_number(GCDisplay *display, MPNumber *z)
 }
 
 
+gboolean
+display_is_number_with_base(GCDisplay *display)
+{
+    MPNumber t;
+    const char *text;
+    const char *sub_digits[] = { "₀", "₁", "₂", "₃", "₄", "₅", "₆", "₇", "₈", "₉", NULL };
+    int i;
+
+    if (display_is_empty(display))
+        return FALSE;
+
+    if (display_is_result(display))
+        return (display->format == BIN || display->format == OCT || display->format == HEX);
+
+    /* See if it has a subscript suffix */
+    text = get_text(display);
+    text += strlen (text);
+    for (i = 0; sub_digits[i] != NULL; i++) {
+        if (strcmp (text - strlen (sub_digits[i]), sub_digits[i]) == 0)
+            return mp_set_from_string(get_text(display), &t) == 0;
+    }
+
+    return FALSE;
+}
+
+
 void
 display_init(GCDisplay *display)
 {
@@ -710,6 +747,27 @@ display_make_number(GCDisplay *display, char *target, int target_len, const MPNu
     }
 }
 
+
+static int
+variable_is_defined(const char *name)
+{
+    char *c, *lower_name;
+
+    lower_name = strdup(name);
+    for (c = lower_name; *c; c++)
+        *c = tolower(*c);
+
+    if (strcmp(lower_name, "rand") == 0 || 
+        strcmp(lower_name, "ans") == 0) {
+        g_free (lower_name);
+        return 1;
+    }
+    g_free (lower_name);
+
+    return register_get_value(name) != NULL;
+}
+
+
 static int
 get_variable(const char *name, MPNumber *z, void *data)
 {
@@ -748,170 +806,18 @@ set_variable(const char *name, const MPNumber *x, void *data)
 }
 
 
-static int
-do_convert(const char *units[][2], const MPNumber *x, const char *x_units, const char *z_units, MPNumber *z)
-{
-    int x_index, z_index;
-    MPNumber x_factor, z_factor;
-    
-    for (x_index = 0; units[x_index][0] != NULL && strcmp(units[x_index][0], x_units) != 0; x_index++);
-    if (units[x_index][0] == NULL)
-        return 0;
-    for (z_index = 0; units[z_index][0] != NULL && strcmp(units[z_index][0], z_units) != 0; z_index++);
-    if (units[z_index][0] == NULL)
-        return 0;
-
-    mp_set_from_string(units[x_index][1], &x_factor);
-    mp_set_from_string(units[z_index][1], &z_factor);
-    mp_multiply(x, &x_factor, z);
-    mp_divide(z, &z_factor, z);
-
-    return 1;
-}
-
-
 static int
 convert(const MPNumber *x, const char *x_units, const char *z_units, MPNumber *z, void *data)
-{
-    const char *length_units[][2] = {
-        {"parsec",     "30857000000000000"},
-        {"parsecs",    "30857000000000000"},
-        {"pc",         "30857000000000000"},
-        {"lightyear",   "9460730472580800"},
-        {"lightyears",  "9460730472580800"},
-        {"ly",          "9460730472580800"},
-        {"au",              "149597870691"},
-        {"nm",                   "1852000"},
-        {"mile",                    "1609.344"},
-        {"miles",                   "1609.344"},
-        {"m",                       "1609.344"},
-        {"kilometer",               "1000"},
-        {"kilometers",              "1000"},
-        {"km",                      "1000"},
-        {"kms",                     "1000"},
-        {"cable",                    "219.456"},
-        {"cables",                   "219.456"},
-        {"cb",                       "219.456"},
-        {"fathom",                     "1.8288"},
-        {"fathoms",                    "1.8288"},
-        {"ftm",                        "1.8288"},
-        {"meter",                      "1"},
-        {"meters",                     "1"},
-        {"m",                          "1"},
-        {"yard",                       "0.9144"},
-        {"yd",                         "0.9144"},
-        {"foot",                       "0.3048"},
-        {"feet",                       "0.3048"},
-        {"ft",                         "0.3048"},
-        {"inch",                       "0.0254"},
-        {"inches",                     "0.0254"},
-        {"centimeter",                 "0.01"},
-        {"centimeters",                "0.01"},
-        {"cm",                         "0.01"},
-        {"cms",                        "0.01"},
-        {"millimeter",                 "0.001"},
-        {"millimeters",                "0.001"},
-        {"mm",                         "0.001"},
-        {"micrometer",                 "0.000001"},
-        {"micrometers",                "0.000001"},
-        {"um",                         "0.000001"},
-        {"nanometer",                  "0.000000001"},
-        {"nanometers",                 "0.000000001"},
-        {NULL, NULL}
-    };
-
-    const char *area_units[][2] = {
-        {"hectare",         "10000"},
-        {"hectares",        "10000"},
-        {"acre",             "4046.8564224"},
-        {"acres",            "4046.8564224"},
-        {"m²",                  "1"},
-        {"cm²",                 "0.001"},
-        {NULL, NULL}
-    };
-
-    const char *volume_units[][2] = {
-        {"cm³",              "1000"},
-        {"gallon",              "3.785412"},
-        {"gallons",             "3.785412"},
-        {"gal",                 "3.785412"},
-        {"litre",               "1"},
-        {"litres",              "1"},
-        {"liter",               "1"},
-        {"liters",              "1"},
-        {"L",                   "1"},
-        {"quart",               "0.9463529"},
-        {"quarts",              "0.9463529"},
-        {"qt",                  "0.9463529"},
-        {"pint",                "0.4731765"},
-        {"pints",               "0.4731765"},
-        {"pt",                  "0.4731765"},
-        {"millilitre",          "0.001"},
-        {"millilitres",         "0.001"},
-        {"milliliter",          "0.001"},
-        {"milliliters",         "0.001"},
-        {"mL",                  "0.001"},
-        {"m³",                  "0.001"},
-        {NULL, NULL}
-    };
-
-    const char *weight_units[][2] = {
-        {"tonne",            "1000"},
-        {"tonnes",           "1000"},
-        {"kilograms",           "1"},
-        {"kilogramme",          "1"},
-        {"kilogrammes",         "1"},
-        {"kg",                  "1"},
-        {"kgs",                 "1"},
-        {"pound",               "0.45359237"},
-        {"pounds",              "0.45359237"},
-        {"lb",                  "0.45359237"},
-        {"ounce",               "0.002834952"},
-        {"ounces",              "0.002834952"},
-        {"oz",                  "0.002834952"},
-        {"gram",                "0.001"},
-        {"grams",               "0.001"},
-        {"gramme",              "0.001"},
-        {"grammes",             "0.001"},
-        {"g",                   "0.001"},
-        {NULL, NULL}
-    };
-    
-    const char *time_units[][2] = {
-        {"year",         "31557600"},
-        {"years",        "31557600"},
-        {"day",             "86400"},
-        {"days",            "86400"},
-        {"hour",             "3600"},
-        {"hours",            "3600"},
-        {"minute",             "60"},
-        {"minutes",            "60"},
-        {"second",              "1"},
-        {"seconds",             "1"},
-        {"s",                   "1"},
-        {"millisecond",         "0.001"},
-        {"milliseconds",        "0.001"},
-        {"ms",                  "0.001"},
-        {"microsecond",         "0.000001"},
-        {"microseconds",        "0.000001"},
-        {"us",                  "0.000001"},
-        {NULL, NULL}
-    };
-    
-    /* See if currency */
+{   
+    /* Update currency if necessary */
+    if (currency_rates_needs_update())
+        currency_download_rates();
     currency_load_rates();
     if (currency_get_index(x_units) >= 0 && currency_get_index(z_units) >= 0)
     {
         currency_convert(x, currency_get_index(x_units), currency_get_index(z_units), z);
         return 1;
     }
-    
-    if (do_convert(length_units, x, x_units, z_units, z) ||
-        do_convert(area_units, x, x_units, z_units, z) ||
-        do_convert(volume_units, x, x_units, z_units, z) ||
-        do_convert(weight_units, x, x_units, z_units, z) ||
-        do_convert(time_units, x, x_units, z_units, z))
-        return 1;
 
     return 0;
 }
@@ -925,6 +831,7 @@ parse(GCDisplay *display, const char *text, MPNumber *z, char **error_token)
     memset(&options, 0, sizeof(options));
     options.wordlen = display->word_size;
     options.angle_units = display->angle_unit;
+    options.variable_is_defined = variable_is_defined;
     options.get_variable = get_variable;
     options.set_variable = set_variable;
     options.convert = convert;
@@ -1104,7 +1011,6 @@ display_do_function(GCDisplay *display, int function, gpointer arg, int cursor_s
     switch (function) {
         case FN_CLEAR:
             display_clear(display);
-            mp_set_from_string("0", ans);
             break;
 
         case FN_SHIFT:
