@@ -1,17 +1,16 @@
-/*  $Header$
+/*  Copyright (c) 1987-2008 Sun Microsystems, Inc. All Rights Reserved.
+ *  Copyright (c) 2008-2009 Robert Ancell
  *
- *  Copyright (c) 1987-2008 Sun Microsystems, Inc. All Rights Reserved.
- *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *           
- *  This program is distributed in the hope that it will be useful, but 
+ *
+ *  This program is distributed in the hope that it will be useful, but
  *  WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  *  General Public License for more details.
- *           
+ *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -35,8 +34,7 @@ enum finc_dialogs {
     FINC_RATE_DIALOG,
     FINC_SLN_DIALOG,
     FINC_SYD_DIALOG,
-    FINC_TERM_DIALOG,
-    FINC_NUM_DIALOGS,
+    FINC_TERM_DIALOG
 };
 
 #endif /* FINANCIAL_H */
