@@ -56,12 +56,10 @@ static ButtonData button_data[] = {
     {"ans",                "ans"},
     {"numeric_point",      "."},
     {"add",                "+"},
-    {"subtract",           "−"},
     {"multiply",           "×"},
     {"divide",             "÷"},
-    {"modulus_divide",     "mod"},
+    {"modulus_divide",     " mod "},
     {"x_pow_y",            "^"},
-    {"exponential",        "×10^"},
     {"percentage",         "%"},
     {"factorial",          "!"},
     {"abs",                "|"},
@@ -75,13 +73,10 @@ static ButtonData button_data[] = {
     {"hyperbolic_cosine",  "cosh"},
     {"hyperbolic_tangent", "tanh"},
     {"inverse",            "⁻¹"},
-    {"base_2",             "₂"},
-    {"base_8",             "₈"},
-    {"base_16",            "₁₆"},
-    {"and",                "and"},
-    {"or",                 "or"},
-    {"xor",                "xor"},
-    {"not",                "not"},
+    {"and",                " AND "},
+    {"or",                 " OR "},
+    {"xor",                " XOR "},
+    {"not",                " NOT "},
     {"integer_portion",    "int"},
     {"fractional_portion", "frac"},
     {"ones_complement",    "ones"},
@@ -92,18 +87,21 @@ static ButtonData button_data[] = {
     {NULL, NULL}
 };
 
+static char *registers[] = {"a", "b", "c", "x", "y", "z", NULL};
+#define MAX_REGISTERS 6
+
 /* Window titles dependant on mode */
 static char *titles[] = {
     /* Translators: The window title when in basic mode */
     N_("Calculator"),
     /* Translators: The window title when in advanced mode */
-    N_("Calculator - Advanced"),
+    N_("Calculator — Advanced"),
     /* Translators: The window title when in financial mode */
-    N_("Calculator - Financial"),
+    N_("Calculator — Financial"),
     /* Translators: The window title when in scientific mode */
-    N_("Calculator - Scientific"),
+    N_("Calculator — Scientific"),
     /* Translators: The window title when in programming mode */
-    N_("Calculator - Programming")
+    N_("Calculator — Programming")
 };
 
 /* The names of each field in the dialogs for the financial functions */
@@ -121,8 +119,8 @@ static char *finc_dialog_fields[][5] = {
     {NULL,        NULL,          NULL,         NULL,         NULL}
 };
 
-#define UI_FILE      PACKAGE_UI_DIR "/gcalctool.ui"
-#define UI_FINC_FILE PACKAGE_UI_DIR "/financial.ui"
+#define UI_FILE      UI_DIR "/gcalctool.ui"
+#define UI_FINC_FILE UI_DIR "/financial.ui"
 
 #define MAXBITS 64      /* Bit panel: number of bit fields. */
 
@@ -173,6 +171,8 @@ typedef struct {
     GtkWidget *superscript_toggle;
     GtkWidget *subscript_toggle;
 
+    gboolean can_super_minus;
+
     /* Labels for popup menus */
     GtkWidget *memory_store_labels[MAX_REGISTERS];
     GtkWidget *memory_recall_labels[MAX_REGISTERS];
@@ -216,8 +216,8 @@ typedef enum {
     CURRENCY_TARGET_LOWER
 } CurrencyTargetRow;
 
-static const char *subscript_digits[] = {"₀", "₁", "₂", "₃", "₄", "₅", "₆", "₇", "₈", "₉"};
-static const char *superscript_digits[] = {"⁰", "¹", "²", "³", "⁴", "⁵", "⁶", "⁷", "⁸", "⁹"};
+static const char *subscript_digits[] = {"₀", "₁", "₂", "₃", "₄", "₅", "₆", "₇", "₈", "₉", NULL};
+static const char *superscript_digits[] = {"⁰", "¹", "²", "³", "⁴", "⁵", "⁶", "⁷", "⁸", "⁹", NULL};
 
 static void load_ui(GtkBuilder *ui, const gchar *filename)
 {
@@ -235,7 +235,7 @@ static void load_ui(GtkBuilder *ui, const gchar *filename)
                                     N_("Error loading user interface"));
     gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog),
                                              /* Translators: Description in UI error dialog when unable to load the UI files. %s is replaced with the error message provided by GTK+ */
-                                             N_("A required file is missing or damaged, please check your installation.\n\n%s"), error->message);
+                                             N_("A required file is missing or damaged. Please check your installation.\n\n%s"), error->message);
     gtk_dialog_add_buttons(GTK_DIALOG(dialog), GTK_STOCK_QUIT, GTK_RESPONSE_ACCEPT, NULL);
 
     gtk_dialog_run(GTK_DIALOG(dialog));
@@ -371,6 +371,18 @@ do_button(int function, gpointer arg)
 {
     GtkTextIter start, end;
     gint cursor_start, cursor_end;
+  
+    /* Can't enter superscript minus after entering digits */
+    if (function == FN_TEXT && strstr("⁰¹²³⁴⁵⁶⁷⁸⁹", (char *)arg) != NULL)
+        X.can_super_minus = FALSE;
+
+    /* Disable super/subscript mode when finished entering */
+    if (function == FN_CALCULATE ||
+        function == FN_CLEAR ||
+        (function == FN_TEXT && strstr("⁻⁰¹²³⁴⁵⁶⁷⁸⁹₀₁₂₃₄₅₆₇₈₉", (char *)arg) == NULL)) {
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(X.subscript_toggle), FALSE);
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(X.superscript_toggle), FALSE);
+    }
 
     if(gtk_text_buffer_get_selection_bounds(X.display_buffer, &start, &end)) {
         cursor_start = gtk_text_iter_get_offset(&start);
@@ -501,10 +513,6 @@ ui_set_display(char *str, int cursor)
     GtkTextIter iter;
     GtkAdjustment *adj;
 
-    // ???
-    if (str[0] == '\0')
-        str = " ";
-
     gtk_text_buffer_set_text(X.display_buffer, str, -1);
 
     if (cursor < 0)
@@ -536,7 +544,6 @@ about_cb(GtkWidget *widget)
 {
     const gchar *authors[] = {
         "Rich Burridge <rich.burridge@sun.com>",
-        "Sami Pietila <sampie@ariana-dsl.utu.fi>",
         "Robert Ancell <robert.ancell@gmail.com>",
         "Klaus Niederkrüger <kniederk@umpa.ens-lyon.fr>",
         NULL
@@ -569,7 +576,7 @@ about_cb(GtkWidget *widget)
                           "name", _("Gcalctool"),
                           "version", VERSION,
                           /* Translators: Copyright notice in the about dialog */
-                          "copyright", _("\xc2\xa9 1986-2008 The Gcalctool authors"),
+                          "copyright", _("\xc2\xa9 1986–2008 The Gcalctool authors"),
                           "license", license,
                           /* Translators: Short description in the about dialog */
                           "comments", _("Calculator with financial and scientific modes."),
@@ -654,7 +661,7 @@ G_MODULE_EXPORT
 void
 recall_menu_cb(GtkMenuItem *menu)
 {
-    do_button(FN_RECALL, g_object_get_data(G_OBJECT(menu), "register_digit"));
+    do_button(FN_RECALL, g_object_get_data(G_OBJECT(menu), "register_id"));
 }
 
 
@@ -669,7 +676,7 @@ finc_activate_cb(GtkWidget *widget) {
     if (finc_dialog_fields[dialog][field+1] == NULL) {
         GtkWidget *dialog_widget;
         dialog_widget = gtk_widget_get_toplevel(widget);
-        if (GTK_WIDGET_TOPLEVEL (dialog_widget)) {
+        if (gtk_widget_is_toplevel (dialog_widget)) {
             gtk_dialog_response(GTK_DIALOG(dialog_widget),
                                 GTK_RESPONSE_OK);
             return;
@@ -811,8 +818,12 @@ G_MODULE_EXPORT
 void
 set_superscript_cb(GtkWidget *widget)
 {
-    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
+       X.can_super_minus = TRUE;
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(X.subscript_toggle), FALSE);
+    }
+    else
+       X.can_super_minus = FALSE;
 }
 
 
@@ -828,22 +839,16 @@ set_subscript_cb(GtkWidget *widget)
 static void
 update_memory_menus()
 {
-    char mstr[MAXLINE], value[MAXLINE];
     int i;
 
-    for (i = 0; i < MAX_REGISTERS; i++) {
-        const char *name, *register_prefix;
+    for (i = 0; registers[i] != NULL; i++) {
+        char value[MAXLINE] = "", mstr[MAXLINE];
+        MPNumber *t;
 
-        name = register_get_name(i);
-
-        /* Translators: R is the short form of register used inter alia in popup menus */
-        register_prefix = _("R");
-
-        display_make_number(&v->display, value, MAXLINE, register_get_value(i));
-        if (name[0] != '\0')
-            SNPRINTF(mstr, MAXLINE, "<span weight=\"bold\">%s%s:</span>    %s [%s]", register_prefix, subscript_digits[i], value, name);
-        else
-            SNPRINTF(mstr, MAXLINE, "<span weight=\"bold\">%s%s:</span>    %s", register_prefix, subscript_digits[i], value);
+        t = register_get_value(registers[i]);
+        if (t)
+            display_make_number(&v->display, value, MAXLINE, t);
+        SNPRINTF(mstr, MAXLINE, "<span weight=\"bold\">%s</span> = %s", registers[i], value);
         gtk_label_set_markup_with_mnemonic(GTK_LABEL(X.memory_store_labels[i]), mstr);
         gtk_label_set_markup_with_mnemonic(GTK_LABEL(X.memory_recall_labels[i]), mstr);
     }
@@ -1001,8 +1006,8 @@ setup_currency_rates ()
         GtkWidget *dialog = gtk_message_dialog_new(NULL, 0,
                                         GTK_MESSAGE_INFO,
                                         GTK_BUTTONS_YES_NO,
-                                        /* Translators: Title of the error dialog when unable to load the UI files */
-                                        N_("You don't have any recent currency rates. Should I download some now?"));
+                                        /* Translators: Title of the error dialog when prompting to download currency rates */
+                                        N_("You don't have any recent currency rates. Should some be downloaded now?"));
         int response = gtk_dialog_run(GTK_DIALOG(dialog));
         gtk_widget_destroy(dialog);
 
@@ -1011,8 +1016,8 @@ setup_currency_rates ()
                 dialog = gtk_message_dialog_new(NULL, 0,
                                                 GTK_MESSAGE_ERROR,
                                                 GTK_BUTTONS_OK,
-                                                /* Translators: Title of the error dialog when unable to load the UI files */
-                                                N_("I could not download any currency rates. You may receive inaccurate results, or you may not receive any results at all."));
+                                                /* Translators: Title of the error dialog when unable to download currency rates */
+                                                N_("Currency rates could not be downloaded. You may receive inaccurate results, or you may not receive any results at all."));
             }
         }
     }
@@ -1099,10 +1104,17 @@ popup_cb(GtkWidget *widget, GdkEventButton *event)
     if (event == NULL) {
         GtkAllocation allocation;
 
-        gdk_window_get_origin(gtk_widget_get_window(widget), &loc.x, &loc.y);
-        gtk_widget_get_allocation(widget, &allocation);
-        loc.x += allocation.x;
-        loc.y += allocation.y;
+        if (X.mode == PROGRAMMING) {
+            gdk_window_get_origin(gtk_widget_get_window(widget), &loc.x, &loc.y);
+            gtk_widget_get_allocation(widget, &allocation);
+            loc.x += allocation.x;
+            loc.y += allocation.y;
+        } else {
+            gdk_window_get_origin(gtk_widget_get_window(X.display_item), &loc.x, &loc.y);
+            gtk_widget_get_allocation(X.display_item, &allocation);
+            loc.x += allocation.x + allocation.width;
+            loc.y += allocation.y + allocation.height;
+	}
         gtk_menu_popup(GTK_MENU(menu), NULL, NULL, menu_pos_func,
                        (gpointer) &loc, 0, gtk_get_current_event_time());
     } else if (event->button == 1) {
@@ -1133,6 +1145,80 @@ digit_cb(GtkWidget *widget, GdkEventButton *event)
 }
 
 
+static void
+do_base(gint base)
+{
+    /* If has a number already in a base, then solve and convert it */
+    if (!display_is_result(&v->display) && display_is_number_with_base(&v->display))
+        do_button(FN_CALCULATE, NULL);
+
+    if (display_is_result(&v->display)) {
+        if (base == 2)
+            display_convert (&v->display, BIN);
+        else if (base == 8)
+            display_convert (&v->display, OCT);
+        else if (base == 16)
+            display_convert (&v->display, HEX);
+        else
+            display_convert (&v->display, DEC);
+    }
+    else {
+        if (base == 2)
+            do_text("₂");
+        else if (base == 8)
+            do_text("₈");
+        else if (base == 16)
+            do_text("₁₆");
+    }
+}
+
+
+static void
+do_exponent()
+{
+    do_text("×10");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(X.superscript_toggle), TRUE);
+}
+
+
+static void
+do_subtract()
+{
+    if (X.can_super_minus) {
+        do_text("⁻");
+        X.can_super_minus = FALSE;
+    }
+    else {
+        do_text("−");
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(X.superscript_toggle), FALSE);
+    }
+}
+
+
+G_MODULE_EXPORT
+void
+base_cb(GtkWidget *widget, GdkEventButton *event)
+{
+    do_base(GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "base")));
+}
+
+
+G_MODULE_EXPORT
+void
+exponent_cb(GtkWidget *widget, GdkEventButton *event)
+{
+    do_exponent();
+}
+
+
+G_MODULE_EXPORT
+void
+subtract_cb(GtkWidget *widget, GdkEventButton *event)
+{
+    do_subtract();
+}
+
+
 G_MODULE_EXPORT
 void
 button_cb(GtkWidget *widget, GdkEventButton *event)
@@ -1157,8 +1243,8 @@ gboolean
 main_window_key_press_cb(GtkWidget *widget, GdkEventKey *event)
 {
     int i, state;
-    const char *conversions[]       = {"-", "*", "/", "\t", NULL};
-    const char *conversion_values[] = {"−", "×", "÷", " ", };
+    const char *conversions[]       = {"*", "/", NULL};
+    const char *conversion_values[] = {"×", "÷", NULL };
 
     /* Only look at the modifiers we use */
     state = event->state & (GDK_CONTROL_MASK | GDK_MOD1_MASK);
@@ -1175,24 +1261,36 @@ main_window_key_press_cb(GtkWidget *widget, GdkEventKey *event)
     if (state == GDK_CONTROL_MASK) {
         switch(event->keyval)
         {
-        case GDK_u:
-            do_text("µ");
+        case GDK_b:
+            do_base(2);
+            return TRUE;
+        case GDK_d:
+            do_base(10);
             return TRUE;
         case GDK_e:
-            do_text("×10^");
+            do_exponent();
             return TRUE;
         case GDK_f:
             do_button(FN_FACTORIZE, NULL);
             return TRUE;
-        case GDK_r:
-            do_text("√");
+        case GDK_h:
+            do_base(16);          
             return TRUE;
         case GDK_i:
             do_text("⁻¹");
             return TRUE;
+        case GDK_o:
+            do_base(8);
+            return TRUE;
         case GDK_p:
             do_text("π");
             return TRUE;
+        case GDK_r:
+            do_text("√");
+            return TRUE;
+        case GDK_u:
+            do_text("µ");
+            return TRUE;
         }
     }
     if (state == GDK_CONTROL_MASK ||
@@ -1289,7 +1387,19 @@ main_window_key_press_cb(GtkWidget *widget, GdkEventKey *event)
     /* Solve */
     if ((event->keyval == GDK_Return && state == 0) ||
         (event->keyval == GDK_KP_Enter && state == 0)) {
-        do_button(FN_CALCULATE, NULL);
+        if (gtk_widget_has_focus(X.display_item)) {
+            do_button(FN_CALCULATE, NULL);
+            return TRUE;
+        }
+        else {
+            return FALSE;
+        }
+    }
+
+    if (state == GDK_CONTROL_MASK && event->keyval == GDK_minus) 
+    {
+        do_text("⁻");
+        X.can_super_minus = FALSE;
         return TRUE;
     }
 
@@ -1298,6 +1408,11 @@ main_window_key_press_cb(GtkWidget *widget, GdkEventKey *event)
 
     // FIXME: event->string deprecated
 
+    if (strcmp(event->string, "-") == 0 || strcmp(event->string, "−") == 0) {
+        do_subtract();
+        return TRUE;
+    }
+
     for (i = 0; conversions[i]; i++) {
         if (strcmp(event->string, conversions[i]) == 0) {
             do_text(conversion_values[i]);
@@ -1318,15 +1433,19 @@ main_window_key_press_cb(GtkWidget *widget, GdkEventKey *event)
     switch(*event->string)
     {
     case '<':
-        button_cb(GET_WIDGET("calc_shift_left_button"), NULL);
+        popup_cb(GET_WIDGET("calc_shift_left_button"), NULL);
         return TRUE;
     case '>':
-        button_cb(GET_WIDGET("calc_shift_right_button"), NULL);
+        popup_cb(GET_WIDGET("calc_shift_right_button"), NULL);
         return TRUE;
     case '\n':
         do_button(FN_CALCULATE, NULL);
         return TRUE;
     }
+  
+    /* Don't override space - it is used in UI */
+    if (event->string[0] == ' ' && !gtk_widget_has_focus(X.display_item))
+        return FALSE;
 
     if (event->string[0] != '\0') {
         do_text(event->string);
@@ -1555,11 +1674,15 @@ create_main_window()
 
     /* Connect super and subscript */
     for (i = 0; i < 10; i++) {
-
         SNPRINTF(name, MAXLINE, "calc_%d_button", i);
         set_string_data(X.ui, name, "calc_subscript_text", subscript_digits[i]);
         set_string_data(X.ui, name, "calc_superscript_text", superscript_digits[i]);
     }
+  
+    /* Set base button data */
+    set_int_data(X.ui, "calc_base_2_button", "base", 2);
+    set_int_data(X.ui, "calc_base_8_button", "base", 8);
+    set_int_data(X.ui, "calc_base_16_button", "base", 16);
 
     /* Connect menus to popup buttons */
     set_data(X.ui, "calc_shift_left_button", "calc_menu", GET_WIDGET("left_shift_popup"));
@@ -1568,15 +1691,15 @@ create_main_window()
     set_data(X.ui, "calc_recall_button", "calc_menu", GET_WIDGET("memory_recall_popup"));
 
     /* Get labels from popup menus */
-    for (i = 0; i < MAX_REGISTERS; i++) {
+    for (i = 0; registers[i]; i++) {
         SNPRINTF(name, MAXLINE, "store_menu_item%d", i);
         widget = GET_WIDGET(name);
-        g_object_set_data(G_OBJECT(widget), "register_id", GINT_TO_POINTER(i));
+        g_object_set_data(G_OBJECT(widget), "register_id", registers[i]);
         X.memory_store_labels[i] = gtk_bin_get_child(GTK_BIN(widget));
 
         SNPRINTF(name, MAXLINE, "recall_menu_item%d", i);
         widget = GET_WIDGET(name);
-        g_object_set_data(G_OBJECT(widget), "register_digit", (gpointer) subscript_digits[i]);
+        g_object_set_data(G_OBJECT(widget), "register_id", registers[i]);
         X.memory_recall_labels[i] = gtk_bin_get_child(GTK_BIN(widget));
     }
 
