@@ -1,19 +1,16 @@
-
-/*  $Header$
- *
- *  Copyright (c) 1987-2008 Sun Microsystems, Inc. All Rights Reserved.
+/*  Copyright (c) 1987-2008 Sun Microsystems, Inc. All Rights Reserved.
  *  Copyright (c) 2008-2009 Robert Ancell
- *           
+ *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *           
- *  This program is distributed in the hope that it will be useful, but 
+ *
+ *  This program is distributed in the hope that it will be useful, but
  *  WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  *  General Public License for more details.
- *           
+ *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -39,18 +36,16 @@ mp_set_from_mp(const MPNumber *x, MPNumber *z)
 void
 mp_set_from_float(float rx, MPNumber *z)
 {
-    int i, k, i2, ib, ie, tp;
-    float rb, rj;
-    
-    i2 = MP_T + 4;
-    
+    int i, k, ib, ie, tp;
+    float rj;
+
     memset(z, 0, sizeof(MPNumber));
 
     /* CHECK SIGN */
-    if (rx < (float) 0.0) {
+    if (rx < 0.0f) {
         z->sign = -1;
         rj = -(double)(rx);
-    } else if (rx > (float) 0.0) {
+    } else if (rx > 0.0f) {
         z->sign = 1;
         rj = rx;
     } else {
@@ -59,28 +54,25 @@ mp_set_from_float(float rx, MPNumber *z)
         return;
     }
 
+    /* INCREASE IE AND DIVIDE RJ BY 16. */
     ie = 0;
-
-    /* INCREASE IE AND DIVIDE RJ BY 16. */    
-    while (rj >= (float)1.0) {
+    while (rj >= 1.0f) {
         ++ie;
-        rj *= (float) 0.0625;
+        rj *= 0.0625f;
     }
-
-    while (rj < (float).0625) {
+    while (rj < 0.0625f) {
         --ie;
-        rj *= (float)16.0;
+        rj *= 16.0f;
     }
 
     /*  NOW RJ IS DY DIVIDED BY SUITABLE POWER OF 16.
      *  SET EXPONENT TO 0
      */
     z->exponent = 0;
-    rb = (float) MP_BASE;
 
     /* CONVERSION LOOP (ASSUME SINGLE-PRECISION OPS. EXACT) */
-    for (i = 0; i < i2; i++) {
-        rj = rb * rj;
+    for (i = 0; i < MP_T + 4; i++) {
+        rj *= (float) MP_BASE;
         z->fraction[i] = (int) rj;
         rj -= (float) z->fraction[i];
     }
@@ -111,50 +103,44 @@ mp_set_from_float(float rx, MPNumber *z)
             tp = 1;
         }
     }
-
-    return;
 }
 
 
 void
 mp_set_from_double(double dx, MPNumber *z)
 {
-    int i, k, i2, ib, ie, tp;
-    double db, dj;
-
-    i2 = MP_T + 4;
+    int i, k, ib, ie, tp;
+    double dj;
 
     memset(z, 0, sizeof(MPNumber));
 
     /* CHECK SIGN */
-    if (dx < 0.)  {
+    if (dx < 0.0)  {
         z->sign = -1;
         dj = -dx;
-    } else if (dx > 0.)  {
+    } else if (dx > 0.0)  {
         z->sign = 1;
         dj = dx;
     } else {
         mp_set_from_integer(0, z);
         return;
-    } 
+    }
 
     /* INCREASE IE AND DIVIDE DJ BY 16. */
     for (ie = 0; dj >= 1.0; ie++)
       dj *= 1.0/16.0;
 
     for ( ; dj < 1.0/16.0; ie--)
-      dj *= 16.;
+      dj *= 16.0;
 
     /*  NOW DJ IS DY DIVIDED BY SUITABLE POWER OF 16
      *  SET EXPONENT TO 0
      */
     z->exponent = 0;
 
-    db = (double) MP_BASE;
-
     /* CONVERSION LOOP (ASSUME DOUBLE-PRECISION OPS. EXACT) */
-    for (i = 0; i < i2; i++) {
-        dj = db * dj;
+    for (i = 0; i < MP_T + 4; i++) {
+        dj *= (double) MP_BASE;
         z->fraction[i] = (int) dj;
         dj -= (double) z->fraction[i];
     }
@@ -185,57 +171,110 @@ mp_set_from_double(double dx, MPNumber *z)
             tp = 1;
         }
     }
-
-    return;
 }
 
 
 void
-mp_set_from_integer(int x, MPNumber *z)
+mp_set_from_integer(int64_t x, MPNumber *z)
 {
+    int i;
+
     memset(z, 0, sizeof(MPNumber));
 
+    if (x == 0) {
+        z->sign = 0;
+        return;
+    }
+
     if (x < 0) {
         x = -x;
         z->sign = -1;
     }
     else if (x > 0)
         z->sign = 1;
-    else
-        z->sign = 0; /* Optimisation for indicating zero */
 
-    z->exponent = 1;
-    z->fraction[0] = x;
-    while (z->fraction[0] >= MP_BASE) {
-        int i;
-        for (i = z->exponent; i >= 0; i--)
-            z->fraction[i] = z->fraction[i-1];
-        z->fraction[0] = z->fraction[1] / MP_BASE;
-        z->fraction[1] = z->fraction[1] % MP_BASE;
+    while (x != 0) {
+        z->fraction[z->exponent] = x % MP_BASE;
+        x = x / MP_BASE;
         z->exponent++;
     }
+    for (i = 0; i < z->exponent / 2; i++) {
+        int t = z->fraction[i];
+        z->fraction[i] = z->fraction[z->exponent - i - 1];
+        z->fraction[z->exponent - i - 1] = t;
+    }
 }
 
 
 void
-mp_set_from_fraction(int i, int j, MPNumber *z)
+mp_set_from_unsigned_integer(uint64_t x, MPNumber *z)
 {
-    mpgcd(&i, &j);
+    int i;
 
-    if (j == 0) {
+    memset(z, 0, sizeof(MPNumber));
+
+    if (x == 0) {
+        z->sign = 0;
+        return;
+    }
+    z->sign = 1;
+
+    while (x != 0) {
+        z->fraction[z->exponent] = x % MP_BASE;
+        x = x / MP_BASE;
+        z->exponent++;
+    }
+    for (i = 0; i < z->exponent / 2; i++) {
+        int t = z->fraction[i];
+        z->fraction[i] = z->fraction[z->exponent - i - 1];
+        z->fraction[z->exponent - i - 1] = t;
+    }
+}
+
+
+void
+mp_set_from_fraction(int64_t numerator, int64_t denominator, MPNumber *z)
+{
+    mp_gcd(&numerator, &denominator);
+
+    if (denominator == 0) {
         mperr("*** J == 0 IN CALL TO MP_SET_FROM_FRACTION ***\n");
         mp_set_from_integer(0, z);
         return;
     }
 
-    if (j < 0) {
-        i = -i;
-        j = -j;
+    if (denominator < 0) {
+        numerator = -numerator;
+        denominator = -denominator;
     }
 
-    mp_set_from_integer(i, z);
-    if (j != 1)
-        mp_divide_integer(z, j, z);
+    mp_set_from_integer(numerator, z);
+    if (denominator != 1)
+        mp_divide_integer(z, denominator, z);
+}
+
+
+void
+mp_set_from_polar(const MPNumber *r, MPAngleUnit unit, const MPNumber *theta, MPNumber *z)
+{
+    MPNumber sin_theta, cos_theta;
+    
+    mp_sin(theta, unit, &sin_theta);
+    mp_cos(theta, unit, &cos_theta);
+    mp_set_from_complex(&cos_theta, &sin_theta, z);
+}
+
+
+void
+mp_set_from_complex(const MPNumber *x, const MPNumber *y, MPNumber *z)
+{
+    //MPNumber i;
+
+    // FIXME: There is some corruption here as i is currently defined to zero but the result does not work
+    //mp_get_i(&i);
+    //mp_multiply(y, &i, z);
+    //mp_add(x, z, z);
+    mp_set_from_mp(x, z);
 }
 
 
@@ -246,54 +285,85 @@ mp_set_from_random(MPNumber *z)
 }
 
 
-int
+int64_t
 mp_cast_to_int(const MPNumber *x)
 {
-    int i, j, x2, xs, ret_val = 0;
-
-    /* RETURN 0 IF X = 0 OR IF NUMBER FRACTION */
-    xs = x->sign;
-    if (xs == 0 || x->exponent <= 0)
+    int i;
+    int64_t z = 0, v;
+  
+    /* |x| <= 1 */
+    if (x->sign == 0 || x->exponent <= 0)
         return 0;
 
-    x2 = x->exponent;
-    for (i = 0; i < x2; i++) {
-        int izs;
-        izs = ret_val;
-        ret_val = MP_BASE * ret_val;
-        if (i < MP_T)
-            ret_val += x->fraction[i];
+    /* Multiply digits together */
+    for (i = 0; i < x->exponent; i++) {
+        int64_t t;
 
-        /* CHECK FOR SIGNS OF INTEGER OVERFLOW */
-        if (ret_val <= 0 || ret_val <= izs)
+        t = z;
+        z = z * MP_BASE + x->fraction[i];
+      
+        /* Check for overflow */
+        if (z <= t)
             return 0;
     }
 
-    /*  CHECK THAT RESULT IS CORRECT (AN UNDETECTED OVERFLOW MAY
-     *  HAVE OCCURRED).
-     */
-    j = ret_val;
-    for (i = x2 - 1; i >= 0; i--) {
-        int j1, kx;
-        
-        j1 = j / MP_BASE;
-        kx = 0;
-        if (i < MP_T)
-            kx = x->fraction[i];
-        if (kx != j - MP_BASE * j1)
+    /* Validate result */
+    v = z;
+    for (i = x->exponent - 1; i >= 0; i--) {
+        int64_t digit;
+
+        /* Get last digit */
+        digit = v - (v / MP_BASE) * MP_BASE;
+        if (x->fraction[i] != digit)
             return 0;
-        j = j1;
+
+        v /= MP_BASE;
     }
-    if (j != 0)
+    if (v != 0)
         return 0;
 
-    /* RESULT CORRECT SO RESTORE SIGN AND RETURN */
-    return xs * ret_val;
+    return x->sign * z;
+}
 
-    /* Old comment about returning zero: */
-    /*  HERE OVERFLOW OCCURRED (OR X WAS UNNORMALIZED), SO
-     *  RETURN ZERO.
-     */
+
+uint64_t
+mp_cast_to_unsigned_int(const MPNumber *x)
+{
+    int i;
+    uint64_t z = 0, v;
+  
+    /* x <= 1 */
+    if (x->sign <= 0 || x->exponent <= 0)
+        return 0;
+
+    /* Multiply digits together */
+    for (i = 0; i < x->exponent; i++) {
+        uint64_t t;
+
+        t = z;
+        z = z * MP_BASE + x->fraction[i];
+      
+        /* Check for overflow */
+        if (z <= t)
+            return 0;
+    }
+
+    /* Validate result */
+    v = z;
+    for (i = x->exponent - 1; i >= 0; i--) {
+        uint64_t digit;
+
+        /* Get last digit */
+        digit = v - (v / MP_BASE) * MP_BASE;
+        if (x->fraction[i] != digit)
+            return 0;
+
+        v /= MP_BASE;
+    }
+    if (v != 0)
+        return 0;
+
+    return z;
 }
 
 
@@ -301,7 +371,7 @@ static double
 mppow_ri(float ap, int bp)
 {
     double pow;
-    
+
     if (bp == 0)
         return 1.0;
 
@@ -311,9 +381,9 @@ mppow_ri(float ap, int bp)
         bp = -bp;
         ap = 1 / ap;
     }
-    
+
     pow = 1.0;
-    for (;;) { 
+    for (;;) {
         if (bp & 01)
             pow *= ap;
         if (bp >>= 1)
@@ -321,7 +391,7 @@ mppow_ri(float ap, int bp)
         else
             break;
     }
-    
+
     return pow;
 }
 
@@ -330,14 +400,13 @@ float
 mp_cast_to_float(const MPNumber *x)
 {
     int i;
-    float rb, rz = 0.0;
-    
-    if (x->sign == 0)
+    float rz = 0.0;
+
+    if (mp_is_zero(x))
         return 0.0;
 
-    rb = (float) MP_BASE;
     for (i = 0; i < MP_T; i++) {
-        rz = rb * rz + (float)x->fraction[i];
+        rz = (float) MP_BASE * rz + (float)x->fraction[i];
 
         /* CHECK IF FULL SINGLE-PRECISION ACCURACY ATTAINED */
         if (rz + 1.0f <= rz)
@@ -345,7 +414,7 @@ mp_cast_to_float(const MPNumber *x)
     }
 
     /* NOW ALLOW FOR EXPONENT */
-    rz *= mppow_ri(rb, x->exponent - i - 1);
+    rz *= mppow_ri((float) MP_BASE, x->exponent - i - 1);
 
     /* CHECK REASONABLENESS OF RESULT */
     /* LHS SHOULD BE <= 0.5, BUT ALLOW FOR SOME ERROR IN ALOG */
@@ -370,13 +439,13 @@ mppow_di(double ap, int bp)
 {
     double pow = 1.0;
 
-    if (bp != 0) { 
+    if (bp != 0) {
         if (bp < 0) {
             if (ap == 0) return(pow);
             bp = -bp;
             ap = 1/ap;
         }
-        for (;;) { 
+        for (;;) {
             if (bp & 01) pow *= ap;
             if (bp >>= 1) ap *= ap;
             else break;
@@ -391,14 +460,13 @@ double
 mp_cast_to_double(const MPNumber *x)
 {
     int i, tm = 0;
-    double d__1, db, dz2, ret_val = 0.0;
+    double d__1, dz2, ret_val = 0.0;
 
-    if (x->sign == 0)
+    if (mp_is_zero(x))
         return 0.0;
 
-    db = (double) MP_BASE;
     for (i = 0; i < MP_T; i++) {
-        ret_val = db * ret_val + (double) x->fraction[i];
+        ret_val = (double) MP_BASE * ret_val + (double) x->fraction[i];
         tm = i;
 
         /* CHECK IF FULL DOUBLE-PRECISION ACCURACY ATTAINED */
@@ -412,7 +480,7 @@ mp_cast_to_double(const MPNumber *x)
     }
 
     /* NOW ALLOW FOR EXPONENT */
-    ret_val *= mppow_di(db, x->exponent - tm);
+    ret_val *= mppow_di((double) MP_BASE, x->exponent - tm - 1);
 
     /* CHECK REASONABLENESS OF RESULT. */
     /* LHS SHOULD BE .LE. 0.5 BUT ALLOW FOR SOME ERROR IN DLOG */
@@ -434,52 +502,49 @@ mp_cast_to_double(const MPNumber *x)
 }
 
 
-void
-mp_cast_to_string(const MPNumber *x, int base, int accuracy, int trim_zeroes, char *buffer, int buffer_length)
+static void
+mp_cast_to_string_real(const MPNumber *x, int base, int accuracy, bool trim_zeroes, bool force_sign, GString *string)
 {
     static char digits[] = "0123456789ABCDEF";
-    MPNumber number, integer_component, fractional_component, MPbase, temp;
+    MPNumber number, integer_component, fractional_component, temp;
     int i, last_non_zero;
-    GString *string;
-    
-    string = g_string_sized_new(buffer_length);
 
     if (mp_is_negative(x))
         mp_abs(x, &number);
     else
         mp_set_from_mp(x, &number);
-   
+
     /* Add rounding factor */
-    mp_set_from_integer(base, &MPbase);
-    mp_xpowy_integer(&MPbase, -(accuracy+1), &temp);
+    mp_set_from_integer(base, &temp);
+    mp_xpowy_integer(&temp, -(accuracy+1), &temp);
     mp_multiply_integer(&temp, base, &temp);
     mp_divide_integer(&temp, 2, &temp);
     mp_add(&number, &temp, &number);
 
     /* Split into integer and fractional component */
     mp_integer_component(&number, &integer_component);
-    mp_fractional_component(&number, &fractional_component);  
+    mp_fractional_component(&number, &fractional_component);
 
     /* Write out the integer component least significant digit to most */
     mp_set_from_mp(&integer_component, &temp);
     do {
         MPNumber t, t2, t3;
-       
+
         mp_divide_integer(&temp, base, &t);
         mp_integer_component(&t, &t);
         mp_multiply_integer(&t, base, &t2);
-       
+
         mp_subtract(&temp, &t2, &t3);
         mp_integer_component(&t3, &t3);
 
         g_string_prepend_c(string, digits[mp_cast_to_int(&t3)]);
-       
+
         mp_set_from_mp(&t, &temp);
     } while (!mp_is_zero(&temp));
 
     last_non_zero = string->len;
     g_string_append_c(string, '.');
-   
+
     /* Write out the fractional component */
     mp_set_from_mp(&fractional_component, &temp);
     for (i = accuracy; i > 0 && !mp_is_zero(&temp); i--) {
@@ -489,7 +554,7 @@ mp_cast_to_string(const MPNumber *x, int base, int accuracy, int trim_zeroes, ch
         mp_multiply_integer(&temp, base, &temp);
         mp_integer_component(&temp, &digit);
         d = mp_cast_to_int(&digit);
-       
+
         g_string_append_c(string, digits[d]);
 
         if(d != 0)
@@ -500,11 +565,72 @@ mp_cast_to_string(const MPNumber *x, int base, int accuracy, int trim_zeroes, ch
     /* Strip trailing zeroes */
     if (trim_zeroes || accuracy == 0)
         g_string_truncate(string, last_non_zero);
+
+    /* Add sign on non-zero values */
+    if (strcmp(string->str, "0") != 0) {
+        if (mp_is_negative(x))
+            g_string_prepend(string, "−");
+        else if (force_sign)
+            g_string_prepend(string, "+");
+    }
+
+    switch(base)
+    {
+    case 2:
+        g_string_append(string, "₂");
+        break;
+    case 8:
+        g_string_append(string, "₈");
+        break;
+    default:
+    case 10:
+        break;
+    case 16:
+        g_string_append(string, "₁₆");
+        break;
+    }
+}
+
+
+void
+mp_cast_to_string(const MPNumber *x, int base, int accuracy, bool trim_zeroes, char *buffer, int buffer_length)
+{
+    MPNumber x_real, x_im;
+    GString *string;
     
-    /* Remove negative sign if the number was rounded down to zero */
-    if (mp_is_negative(x) && strcmp(string->str, "0") != 0)
-        g_string_prepend(string, "−");
+    string = g_string_sized_new(buffer_length);
     
+    mp_real_component(x, &x_real);
+    mp_imaginary_component(x, &x_im);
+    
+    mp_cast_to_string_real(&x_real, base, accuracy, trim_zeroes, FALSE, string);
+    if (mp_is_complex(x)) {
+        GString *s;
+        gboolean force_sign = TRUE;
+        
+        if (strcmp(string->str, "0") == 0) {
+            g_string_assign(string, "");
+            force_sign = FALSE;
+        }
+
+        s = g_string_sized_new(buffer_length);
+        mp_cast_to_string_real(&x_im, 10, accuracy, trim_zeroes, force_sign, s);
+        if (strcmp(s->str, "1") == 0) {
+            g_string_append(string, "i");
+        }
+        else if (strcmp(s->str, "+1") == 0) {
+            g_string_append(string, "+i");
+        }
+        else if (strcmp(s->str, "−1") == 0) {
+            g_string_append(string, "−i");
+        }
+        else {
+            g_string_append(string, s->str);
+            g_string_append(string, "i");
+        }
+        g_string_free(s, TRUE);
+    }
+
     // FIXME: Check for truncation
     strncpy(buffer, string->str, buffer_length);
     g_string_free(string, TRUE);
@@ -572,41 +698,56 @@ char_val(char **c, int base)
     }
     if (value >= base)
        return -1;
-    
+
     *c += offset;
 
     return value;
 }
 
 
-int
-mp_set_from_string(const char *str, int base, MPNumber *z)
+static int
+ends_with(const char *start, const char *end, const char *word)
 {
-    int i, negate = 0, multiplier = 0;
+    size_t word_len = strlen(word);
+
+    if (word_len > end - start)
+        return 0;
+
+    return strncmp(end - word_len, word, word_len) == 0;
+}
+
+
+bool
+mp_set_from_string(const char *str, MPNumber *z)
+{
+    int i, base, negate = 0, multiplier = 0, base_multiplier = 1;
     const char *c, *end;
     gboolean has_fraction = FALSE;
-    
-    const char *base_suffixes[] = {"₂", "₈", "₁₆", NULL};
-    int base_values[]           = {2, 8, 16, 10};
+
+    const char *base_digits[]   = {"₀", "₁", "₂", "₃", "₄", "₅", "₆", "₇", "₈", "₉", NULL};
     const char *fractions[]     = {"½", "⅓", "⅔", "¼", "¾", "⅕", "⅖", "⅗", "⅘", "⅙", "⅚", "⅛", "⅜", "⅝", "⅞", NULL};
     int numerators[]            = { 1,   1,   2,   1,   3,   1,   2,   3,   4,   1,   5,   1,   3,   5,   7};
     int denominators[]          = { 2,   3,   3,   4,   4,   5,   5,   5,   5,   6,   6,   8,   8,   8,   8};
-    const char *si_suffixes[]   = {"T", "G", "M", "k", "d", "c", "m", "u", "µ", "n", "p", "f", NULL};
-    int si_multipliers[]        = { 12,   9,   6,   3,  -1,  -2,  -3,  -6,  -6,  -9, -12, -15};
-    
+
     /* Find the base */
     end = str;
     while (*end != '\0')
         end++;
-    if (base < 0) {
-        for (i = 0; base_suffixes[i] != NULL; i++) {
-            if (end - strlen(base_suffixes[i]) < str)
-                continue;
-            if (strcmp(end - strlen(base_suffixes[i]), base_suffixes[i]) == 0)
+    base = 0;
+    while (1) {
+        for (i = 0; base_digits[i] != NULL; i++) {
+            if (ends_with(str, end, base_digits[i])) {
+                base += i * base_multiplier;
+                end -= strlen(base_digits[i]);
+                base_multiplier *= 10;
                 break;
+            }
         }
-        base = base_values[i];
+        if (base_digits[i] == NULL)
+            break;
     }
+    if (base_multiplier == 1)
+        base = 10;
 
     /* Check if this has a sign */
     c = str;
@@ -623,38 +764,30 @@ mp_set_from_string(const char *str, int base, MPNumber *z)
     /* Convert integer part */
     mp_set_from_integer(0, z);
     while ((i = char_val((char **)&c, base)) >= 0) {
+        if (i > base)
+            return true;
         mp_multiply_integer(z, base, z);
         mp_add_integer(z, i, z);
     }
-    
+
     /* Look for fraction characters, e.g. ⅚ */
     for (i = 0; fractions[i] != NULL; i++) {
-        if (end - strlen(fractions[i]) < str)
-            continue;
-        if (strcmp(end - strlen(fractions[i]), fractions[i]) == 0)
+        if (ends_with(str, end, fractions[i])) {
+            end -= strlen(fractions[i]);
             break;
+        }
     }
     if (fractions[i] != NULL) {
         MPNumber fraction;
         mp_set_from_fraction(numerators[i], denominators[i], &fraction);
         mp_add(z, &fraction, z);
     }
-    
+
     if (*c == '.' || *c == ',') {
         has_fraction = TRUE;
         c++;
-    } else {
-        for (i = 0; si_suffixes[i] != NULL; i++) {
-            if (strncmp(c, si_suffixes[i], strlen(si_suffixes[i])) == 0)
-                break;
-        }
-        if (si_suffixes[i] != NULL) {
-            has_fraction = TRUE;
-            multiplier = si_multipliers[i];
-            c += strlen(si_suffixes[i]);
-        }
     }
-   
+
     /* Convert fractional part */
     if (has_fraction) {
         MPNumber numerator, denominator;
@@ -669,53 +802,20 @@ mp_set_from_string(const char *str, int base, MPNumber *z)
         mp_divide(&numerator, &denominator, &numerator);
         mp_add(z, &numerator, z);
     }
-   
-    /* Convert exponential part */
-    if (*c == 'e' || *c == 'E') {
-        int negate = 0;
-        MPNumber MPbase, MPexponent, temp;
 
-        c++;
-
-        /* Get sign */
-        if (*c == '-') {
-            negate = 1;
-            c++;
-        } else if (strncmp(c, "−", strlen("−")) == 0) {
-            negate = 1;
-            c += strlen("−");
-        } else if (*c == '+') {
-            c++;
-        }
-
-        /* Get magnitude */
-        mp_set_from_integer(0, &MPexponent);
-        while ((i = char_val((char **)&c, base)) >= 0) {
-            mp_multiply_integer(&MPexponent, base, &MPexponent);
-            mp_add_integer(&MPexponent, i, &MPexponent);
-        }
-        if (negate) {
-            mp_invert_sign(&MPexponent, &MPexponent);
-        }
-
-        mp_set_from_integer(base, &MPbase);       
-        mp_xpowy(&MPbase, &MPexponent, &temp);
-        mp_multiply(z, &temp, z);
-    }
-   
     if (c != end) {
-        return 1;
+        return true;
     }
-    
+
     if (multiplier != 0) {
         MPNumber t;
         mp_set_from_integer(10, &t);
         mp_xpowy_integer(&t, multiplier, &t);
         mp_multiply(z, &t, z);
     }
- 
+
     if (negate == 1)
         mp_invert_sign(z, z);
-    
-    return 0;
+
+    return false;
 }
