@@ -86,7 +86,7 @@ test(char *expression, char *expected, int expected_error)
     if(error == 0) {
         mp_cast_to_string(&result, base, 9, 1, result_str, 1024);
         if(expected_error != 0)
-            fail("'%s' -> %s, expected error %d", expression, result_str, expected_error);
+            fail("'%s' -> %s, expected error %s", expression, result_str, error_code_to_string(expected_error));
         else if(strcmp(result_str, expected) != 0)
             fail("'%s' -> '%s', expected '%s'", expression, result_str, expected);
         else
@@ -102,6 +102,40 @@ test(char *expression, char *expected, int expected_error)
 }
 
 
+static void
+test_conversions()
+{
+    memset(&options, 0, sizeof(options));
+    base = 10;
+    options.wordlen = 32;
+    options.angle_units = MP_DEGREES;
+
+    /* Length */
+    test("1 meter in mm", "1000", 0);  
+    test("1m in mm", "1000", 0);
+    test("1 inch in cm", "2.54", 0);
+
+    /* Area */
+    test("1m² in mm²", "1000000", 0);
+  
+    /* Volume */
+    test("1m³ in mm³", "1000000000", 0);  
+
+    /* Weight */
+    test("1 kg in pounds", "2.204622622", 0);
+  
+    /* Time */
+    test("1 minute in seconds", "60", 0);
+}
+
+
+int
+variable_is_defined(const char *name)
+{
+    return strcmp (name, "x") == 0 || strcmp (name, "y") == 0;
+}
+
+
 int
 get_variable(const char *name, MPNumber *z, void *data)
 {
@@ -122,14 +156,14 @@ set_variable(const char *name, const MPNumber *x, void *data)
 {
 }
 
-
 void
-test_parser()
+test_equations()
 {
     memset(&options, 0, sizeof(options));
     base = 10;
     options.wordlen = 32;
     options.angle_units = MP_DEGREES;
+    options.variable_is_defined = variable_is_defined;  
     options.get_variable = get_variable;
     options.set_variable = set_variable;
 
@@ -220,18 +254,30 @@ test_parser()
     test("y", "3", 0);
     test("z", "", PARSER_ERR_UNKNOWN_VARIABLE);
     test("2y", "6", 0);
-    test("y2", "", PARSER_ERR_UNKNOWN_FUNCTION);
+    test("y2", "", PARSER_ERR_INVALID);
+    test("y 2", "", PARSER_ERR_INVALID);
+    test("2z", "", PARSER_ERR_UNKNOWN_VARIABLE);  
+    test("z2", "", PARSER_ERR_UNKNOWN_VARIABLE);
+    test("z 2", "", PARSER_ERR_UNKNOWN_VARIABLE);
+    test("z(2)", "", PARSER_ERR_UNKNOWN_VARIABLE);
     test("y²", "9", 0);
     test("2y²", "18", 0);
+    test("x×y", "6", 0);
     test("xy", "6", 0);
     test("yx", "6", 0);
     test("2xy", "12", 0);
-    //test("x²y", "12", 0);
-    //test("xy²", "18", 0);
+    test("x²y", "12", 0);
+    test("xy²", "18", 0);
     test("(xy)²", "36", 0);
-    //test("2x²y", "24", 0);
-    //test("2xy²", "36", 0);
-    //test("2x²y²", "72", 0);
+    test("2x²y", "24", 0);
+    test("2xy²", "36", 0);
+    test("2x²y²", "72", 0);
+    test("x²yx²y", "144", 0);
+    test("x³+2x²−5", "11", 0);
+    test("2(x+3y)", "22", 0);
+    test("x(x+3y)", "22", 0);
+    test("(x+3y)(2x-4y)", "−88", 0);
+    test("2x²+2xy−12y²", "−88", 0);
 
     test("π", "3.141592654", 0);
     test("e", "2.718281828", 0);
@@ -634,6 +680,7 @@ unittest()
 {
     test_mp();
     test_numbers();
-    test_parser();
+    test_conversions();
+    test_equations();
     exit(fails > 0 ? 1 : 0);
 }
